use Game::Quest::Entity;

use Game::Quest::EntityNoUpdate;
use Game::Quest::EntityLeftUpdate;
use Game::Quest::EntityRightUpdate;
use Game::Quest::EntityUpUpdate;
use Game::Quest::EntityDownUpdate;

use Game::Quest::ImageLibrary;

class Player {

	has $!ddx;
	has $!ddy;

	has $!update-obj;

	has $!leftimagelibrary;
	has $!rightimagelibrary;
	has $!upimagelibrary;
	has $!downimagelibrary;

	submethod BUILD(:$x, :$y, :$ddx, :$ddy) {
		.init($x,$y, $ddx, $ddy);
	}

	multi method init($x, $y, $ddx, $ddy) {
		.x = $x;
		.y = $y;

		.ddx = $ddx;
		.ddy = $ddy;

		.update-obj = EntityNoUpdate.new(.ddx, .ddy);
		.update-obj.update;

		.leftimagelibrary = ImageLibrary.new; 
		.rightimagelibrary = ImageLibrary.new; 
		.upimagelibrary = ImageLibrary.new; 
		.downimagelibrary = ImageLibrary.new; 
	}

	### adding image loops for player's direction 
	multi method add-left-image($filename) {
		.leftimagelibrary.add-image($filename);
	}	

	multi method add-right-image($filename) {
		.rightimagelibrary.add-image($filename);
	}	

	multi method add-up-image($filename) {
		.upimagelibrary.add-image($filename);
	}	

	multi method add-down-image($filename) {
		.downimagelibrary.add-image($filename);
	}	

	### fetch this after an update (see below), to blit the player
	### at xy
	multi method get-xy {
		my ($dx,$dy) = .update-obj.get-dxdy;
		return (.x + $dx, .y + $dy);
	}

	### methods for keyboard functionality :
	multi method move-not {
		.update-obj = EntityNoUpdate.new(.ddx, .ddy);
		.update-obj.update;
	}

	multi method move-left {
		.update-obj = EntityLeftUpdate.new(.ddx, .ddy);
		.update-obj.update;
	}

	multi method move-right {
		.update-obj = EntityRightUpdate.new(.ddx, .ddy);
		.update-obj.update;
	}

	multi method move-up {
		.update-obj = EntityUpUpdate.new(.ddx, .ddy);
		.update-obj.update;
	}

	multi method move-down {
		.update-obj = EntityDownUpdate.new(.ddx, .ddy);
		.update-obj.update;
	}


	
}
